###################################################################################################
# _blobfields.py
#
# Implementation of classes MyFile, MyImage, etc. (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from OFS.Image import Image, File
import string
import time
# Product Imports.
from _globals import *
import _fileutil


###################################################################################################
# createBlobField
###################################################################################################
def createBlobField(datatype):
  if datatype in ['myimage']:
    return MyImage(id='',title='',file='')
  elif datatype in ['myfile']:
    return MyFile(id='',title='',file='')
  else:
    return None

###################################################################################################
# synchronizeBlobFields
###################################################################################################
def synchronizeBlobFields(self):
  #++print "[%s.synchronizeBlobFields]:"%(self.meta_type)

  # Remove existing DTML Blob-Methods.
  title = 'blob_html'
  ids = []
  for ob in self.objectValues(['DTML Method']):
    if ob.title == title:
      ids.append(ob.id())
  try:
    #++ print "[%s.synchronizeBlobFields]: Delete %s(DTML Method)..."%(self.meta_type,str(ids))
    self.manage_delObjects(ids)
  except:
    pass
    
  # Create new DTML Blob-Methods.
  for key in self.getObjAttrs().keys():
    obj_attr = self.getObjAttr(key)
    datatype = self.getObjDatatype(obj_attr)
    if datatype in ['myfile','myimage']:
      for s_lang in self.getLanguages():
        for s_preview in [None,'preview']:
          req = {}
          req['lang'] = s_lang
          req['preview'] = s_preview
          myobj = self.getObjProperty(key,req)
          if myobj is not None:
            filename = _fileutil.extractFileName(myobj.filename)
            filename = string.replace(filename,'%','')
            filename = string.replace(filename,' ','')
  	    raw = '<dtml-var "blob_html(\'%s\',REQUEST,RESPONSE)">'%key
            try:
              self.manage_addDTMLMethod(filename,title,raw)
            except:
              pass
            for ob in self.objectValues(['DTML Method']):
              if ob.id() == filename: 
                zmethod = ob
                #++ print "[synchronizeBlobFields]: %s"%zmethod.absolute_url()


###################################################################################################

class MyBlob:

    def download(self, sender, folder):
      sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url()):]
      sFilename = '%s/%s/%s'%(folder,sFilepath,_fileutil.extractFileName(self.filename))
      _fileutil.exportObj(self,sFilename)


    src__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def src(self, sender, key, REQUEST):
      
      filename = _fileutil.extractFileName(self.filename) # = 'blob_html'
      filename = string.replace(filename,'%','')
      filename = string.replace(filename,' ','')
      
      s = '%s/%s/blob_html'%(sender.aq_parent.absolute_url(),sender.id)
      for ob in sender.objectValues(['DTML Method']):
        if ob.id() == filename: 
          s = '%s/%s/%s'%(sender.aq_parent.absolute_url(),sender.id,filename)
      
      s = s + '?key=%s'%key
      s = s + '&lang=%s'%REQUEST['lang']
      if isPreviewRequest(REQUEST):
        s = s + "&preview=preview"
        s = s + "&time=" + str(time.time())
      return s

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.xmlGetTagName: 
    #
    #	Returns <XML> Tag-Name.
    # ---------------------------------------------------------------------------------------------
    def xmlGetTagName(self):
      return 'data'

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getDataSizeStr: 
    #
    #	Display string for file-size.
    # ---------------------------------------------------------------------------------------------
    def getDataSizeStr(self):
      s = ''
      data = getattr(self,'data','')
      if data is not None:
        s = getDataSizeStr(float(len(data)))
      return s

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getMimeType: 
    #
    #	MIME-Type.
    # ---------------------------------------------------------------------------------------------
    def getMimeType(self):
      return self.content_type

###################################################################################################

class MyImage(MyBlob,Image):

    __obj_attrs__ = ['size','width','height','content_type','data','filename']
    
    def __str__(self):
      return '{{MyImage}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyImage._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self, ob=None):
        if not ob:
          ob = self
        oClone = MyImage(id='',title='',file='')
        for attr in self.__obj_attrs__:
          if hasattr(ob,attr):
            setattr(oClone,attr,getattr(ob,attr))
        return oClone

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.upload:
    # ---------------------------------------------------------------------------------------------
    def upload(self, sender, folder):
        sFilename = "%s/%s"%(folder,self.filename)
        sFilename = _fileutil.getOSPath(sFilename)
        print '[upload]: sFilename="%s"'%sFilename
        # Backup properties (otherwise manage_upload sets it).
        width = self.width
        height = self.height
        content_type = self.content_type
        filename = _fileutil.extractFileName(self.filename)
        # Read file to ZODB.
        f = open(sFilename,'rb')
        self.manage_upload(f.read(),None)
        f.close()
        # Restore properties.
        self.width = width
        self.height = height
        self.content_type = content_type
        self.filename = filename

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender):
        sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url()):]
        sFilename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
        xml = '\n<%s'%self.xmlGetTagName()
        xml = xml + ' width="%s"'%str(getattr(self,'width',''))
        xml = xml + ' height="%s"'%str(getattr(self,'height',''))
        xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
        xml = xml + ' filename="%s%s"'%(sFilepath,sFilename)
        xml = xml + '>'
        xml = xml + '</%s>'%self.xmlGetTagName()
        return xml

    getWidth__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getWidth(self):
      return self.width

    getHeight__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getHeight(self):
      return self.height

###################################################################################################

class MyFile(MyBlob,File):

    __obj_attrs__ = ['size','content_type','data','filename']

    def __str__(self):
      return '{{MyFile}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyFile._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self, ob=None):
        if not ob:
            ob = self
        oClone = MyFile(id='',title='',file='')
        for attr in self.__obj_attrs__:
            if hasattr(ob,attr):
                setattr(oClone,attr,getattr(ob,attr))
        return oClone

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.upload:
    # ---------------------------------------------------------------------------------------------
    def upload(self, sender, folder):
        sFilename = "%s/%s"%(folder,self.filename)
        sFilename = _fileutil.getOSPath(sFilename)
        print '[upload]: sFilename="%s"'%sFilename
        # Backup properties (otherwise manage_upload sets it).
        content_type = self.content_type
        filename = _fileutil.extractFileName(self.filename)
        # Read file to ZODB.
        f = open(sFilename,'rb')
        self.manage_upload(f.read(),None)
        f.close()
        # Restore properties.
        self.content_type = content_type
        self.filename = filename

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender):
        sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url()):]
        sFilename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
        xml = '\n<%s'%self.xmlGetTagName()
        xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
        xml = xml + ' filename="%s%s"'%(sFilepath,sFilename)
        xml = xml + '>'
        xml = xml + '</%s>'%self.xmlGetTagName()
        return xml

###################################################################################################
