###################################################################################################
# _fileutil.py
#
# Implementation of file utilities.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from App.Common import package_home
import os
import stat
import string
import tempfile


# -------------------------------------------------------------------------------------------------
#  _fileutil.importZexp:
#
# Import file from specified path.
# -------------------------------------------------------------------------------------------------
def importZexp(self, path, filename):
  src_filename = path + filename
  dst_filename = INSTANCE_HOME + '/import/' + filename
  if src_filename != dst_filename:
    cpFile(src_filename,dst_filename)
  self.manage_importObject(filename)
  rmFile(dst_filename)


# -------------------------------------------------------------------------------------------------
#  _fileutil.extractFileName:
#
#  Extract filename from path.
# -------------------------------------------------------------------------------------------------
def extractFileName(path):
  items = string.split(getOSPath(path),os.sep)
  lastitem = items[len(items)-1]
  return lastitem


# -------------------------------------------------------------------------------------------------
#  _fileutil.getOSPath:
# 
#  Return path with OS separators.
# -------------------------------------------------------------------------------------------------
def getOSPath(path, sep=None):
  if sep is None: sep = os.sep
  path = string.replace(path,'\\',sep)
  path = string.replace(path,'/',sep)
  for ch in range(0,255):
    if ch < 32 or ch in [34,60,62] or ch > 126: 
      path = string.replace(path,chr(ch),'')
  return path


# -------------------------------------------------------------------------------------------------
#  _fileutil.getFilePath:
#
#  Extract filepath from path (cut-off filename).
#  IN:	path
#  OUT:	filepath
# -------------------------------------------------------------------------------------------------
def getFilePath(path):
  items = string.split(getOSPath(path),os.sep)
  rtn = ''
  for i in range(0,len(items)-1):
    rtn = rtn + items[i] + os.sep
  if rtn[-1] == os.sep:
    rtn = rtn[:-1]
  return rtn


###############################################################################################    
# getFileExt(path)
# 
# Extract fileextension from path.
# IN:	path
# OUT:	extension
###############################################################################################    
def getFileExt(path):
    items = string.split(path,'.')
    lastitem = items[len(items)-1]
    return lastitem


###############################################################################################    
# getFileBase(path)
###############################################################################################    
def getFileBase(path):
    items = string.split(getOSPath(path),os.sep)
    lastitem = items[len(items)-1]
    items = string.split(lastitem,'.')    
    return items[0]


###############################################################################################    
# findExtension(extension, path)
#
# Searches path and all subdirectories for file with extension and returns complete filepath.
# Returns None if no file with specified extension exists.
###############################################################################################
def findExtension(extension, path):
    rtn = None
    path = getOSPath(path)
    for file in os.listdir(path):
        filepath = path + os.sep + file 
        if string.lower(getFileExt(file)) == extension:
            return filepath
        else:
            mode = os.stat(filepath)[stat.ST_MODE]
            if stat.S_ISDIR(mode):
                rtn = findExtension(extension,filepath)
    return rtn


###############################################################################################    
# cpFile:
#
# Copies src to dst.
###############################################################################################
def cpFile(src, dst):
  #++ print "[cpFile]: %s to %s"%(src,dst)
  f_src = open(getOSPath(src),'rb')
  f_dst = open(getOSPath(dst),'wb')
  f_dst.write(f_src.read())
  f_dst.close()
  f_src.close()


###############################################################################################    
# rmFile:
#
# Removes file.
###############################################################################################
def rmFile(file):
  #++ print "[rmFile]: %s"%(file)
  os.remove(getOSPath(file))


###############################################################################################    
# rmFilePath:
#
# Removes path and all its subdirectories.
###############################################################################################
def rmFilePath(sPath):
  sPath = getOSPath(sPath)
  for sFileName in os.listdir(sPath):
      sFilePath = sPath + os.sep + sFileName
      mode = os.stat(sFilePath)[stat.ST_MODE]
      if stat.S_ISDIR(mode):
        rmFilePath(sFilePath)
      else: 
        rmFile(sFilePath)
  os.rmdir(sPath)


###############################################################################################    
# extractZipArchive(zipfile)
#
# Unpack ZIP-Archive.
###############################################################################################
def extractZipArchive(sZipFilename):
    
    # DOS, Win32:
    # -----------
    # PKZIP -extract -direct zipfile destinationpath : Extract files from .ZIP
    if os.name in ['nt','dos']:
      command = package_home(globals()) + '/bin/pkzip/pkzip25.exe -extract -direct %s %s'%(sZipFilename,getFilePath(sZipFilename))
      command = getOSPath(command)
    # UNIX, Linux, Solaris:
    # ---------------------
    # UNZIP -d zipfile : Extract files from .ZIP
    #   -d extract files into exdir
    else:
      command = 'unzip -d %s %s'%(getFilePath(sZipFilename),sZipFilename)
    
    # Execute command.
    # ----------------
    os.system(command)


###############################################################################################    
# buildZipArchive(sFilelist)
#
# Create ZIP-Archive.
###############################################################################################
def buildZipArchive(sFilelist):
    
    # Create temporary zip-archive.
    # -----------------------------
    sTmpZipFilename = tempfile.mktemp() + '.zip'
    
    # DOS, Win32:
    # -----------
    # PKZIP
    #   -add             Add files
    #   -rec             Recurse subdirectories
    #   -dir=relative    Store path information relative to the 
    #                    specified directory
    if os.name in ['nt','dos']:
        command = package_home(globals()) + '/bin/pkzip/pkzip25.exe' \
                + ' -add -rec -dir=relative' \
                + ' ' + sTmpZipFilename \
                + ' ' + sFilelist
        command = getOSPath(command)
        
    # UNIX, Linux, Solaris:
    # ---------------------
    # ZIP
    #   -r	recurse into directories
    #   -R	PKZIP recursion (see manual)
    else:
        sFilelist = string.replace(sFilelist, '*.*', '*')
        command = 'zip' \
                + ' -r' \
                + ' ' + sTmpZipFilename \
                + ' ' + sFilelist
                
    # Execute command.
    # ----------------
    os.system(command)
    
    # Read content zip-archive as return value.
    # -----------------------------------------
    fTmpZip = open(sTmpZipFilename,'rb')
    sRtn = fTmpZip.read()
    fTmpZip.close()
    
    # Remove temporary zip-archive.
    # -----------------------------
    os.remove(sTmpZipFilename)
    
    # Return content of zip-archive.
    # ------------------------------
    return sRtn


    
###############################################################################################    
# exportObj:
###############################################################################################    
def exportObj(obj, filename):
    """exportObj"""
    
    # --- Try to create directory-tree.
    filename = getOSPath(filename)
    filepath = getFilePath(filename)
    try:
      os.makedirs(filepath)
      #++ print '[_fileutil.exportObj]: Created path %s'%(filepath)
    except:
      pass # do nothing
        
    # --- Get object data.   
    try: # ImageFile        
      f = open(obj.path,'rb')
      data = f.read()
      f.close()
    except:
      try: # Image
        data = obj.data
      except: 
        try:
          data = obj.raw # DTML Method
        except:
          try:
	    data = obj.read() # REQUEST.enctype multipart/form-data
	  except:
            data = str(obj)

    # --- Save to file.
    #++ print '[_fileutil.exportObj]: filename = %s'%(filename)
    if data:
        objfile = open( filename, 'wb' )
        if type(data) is type(''):
            objfile.write( data ) 
        else:
            while data is not None:
                objfile.write(data.data)
                data=data.next
        objfile.close()
    return

###############################################################################################    
