###################################################################################################
# zmsfile.py
#
# Implementation of class ZMSFile (see below).
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile, ImageFile, Persistent
from OFS.Folder import Folder
from sys import *
import string 
import urllib
# Product imports.
from _metadata import Metadata
from zmsobject import ZMSObject
import _globals
import _fileutil
import _mimetype
import _blobfields


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSFileForm = HTMLFile('manage_addzmsfileform', globals()) 
def manage_addZMSFile(self, lang, manage_lang, _sort_id, REQUEST):
  """ manage_addZMSFile """
    
  if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
    
    ##### Create ####
    obj = ZMSFile(self.getNewId(),_sort_id+1)
    self._setObject(obj.id, obj)
        
    for oItem in self.objectValues():
      if oItem.id == obj.id:
        ##### Object State ####
        oItem.setObjStateNew(REQUEST)
        ##### Init Properties ####
        for s_lang in self.getLanguages():
          oItem.setObjProperty('active',_globals.true,s_lang)
        oItem.manage_changeProperties(lang,manage_lang,REQUEST)
        ##### VersionManager ####
        oItem.__autoCommit__(REQUEST)

    ##### Normalize Sort-IDs ####
    self.normalizeSortIds()
        
    if REQUEST.RESPONSE:
      message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
      REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.absolute_url(),lang,manage_lang,urllib.quote(message),obj.id))
                
  else:         
    if REQUEST.RESPONSE:
      REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s'%(self.absolute_url(),lang,manage_lang))



###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################
class ZMSFile(ZMSObject, Folder, _mimetype.MimeType, Metadata): 
        
    # Properties.
    # -----------
    meta_type = 'ZMSFile'
    icon = "misc_/zms/zmsfile.gif"
    icon_disabled = 'misc_/zms/zmsfile_disabled.gif'

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',    'action': 'manage_main'},
	{'label': 'TAB_PREVIEW', 'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main','manage_menu',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_changeProperties',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		('Import/Export objects', ('manage_importexport','manage_import','manage_export',) ),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        'sort_id':{'datatype':'string','native':1},
        'change_uid':{'datatype':'LangString','lang_inherit':-1},
        'change_dt':{'datatype':'LangDate','lang_inherit':-1},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':0},
        'title':{'datatype':'LangString'},
        'titleshort':{'datatype':'LangString'},
        'file':{'datatype':'LangMyFile'},
        'counter':{'datatype':'LangInt','default':0,'lang_inherit':-1},
        # Meta-Dictionaries
        '$metadict':{'datatype':'MetaDict'},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/zmsfile/manage_main',globals()) 


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSFile.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
        sMessage = ''
        
        # Process
        try:
            delattr(self,'format')
        except:
            pass
        
        # Process ZMSFile.
        try: delattr(self,'file')
        except: pass
        try: delattr(self,'src')
        except: pass

        # ---------------------------------------------------------------------
        # 26.11.2001 - 0.8 to 0.9: 
        # Meta-Dictionaries multilingual
        # ---------------------------------------------------------------------
        for s_attr in self.getMetadictAttrs(self.meta_type):
          if hasattr(self,s_attr):
            for s_lang in self.getLanguages():
              setattr(self,'%s_%s'%(s_attr,s_lang),getattr(self,s_attr))
            try: delattr(self,s_attr)
            except: pass
            
        # ---------------------------------------------------------------------
        # 11.01.2002 - 0.9 to 1.0: 
        # Blobs.
        # ---------------------------------------------------------------------
        _blobfields.synchronizeBlobFields(self)

        # Process super-objects.
        sMessage = sMessage + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        sMessage = sMessage + self.metadatarecurse_updateVersion(lang,manage_lang,REQUEST)
        
        # Return with message.
        return sMessage


    # -------------------------------------------------------------------------------------------------
    #	ZMSFile.isPageElement:
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self): return 1
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSFile.getLinkList: 
    #
    #	Returns file as link-URL.
    # ---------------------------------------------------------------------------------------------
    def getLinkList(self,REQUEST, allow_none=0):
        rtn = []
        append = 1
        append = append and self.getObjectDisplay(REQUEST) # Object has been committed.
        append = append and self.isActive(REQUEST) # Object is active.
        append = append and not self.getObjProperty('attr_dc_type',REQUEST) in ['Resource'] # Object is not resource or teaser.
	if append:         
	  dct = {}
	  dct['internal'] = 1
	  dct['src'] = self
	  dct['dst'] = self
	  if self.getFile(REQUEST) is None:
	    dct['url'] = ''
	  else:
	    dct['url'] = self.getFile(REQUEST).src(self,'file',REQUEST)
	  dct['title'] = self.getTitle(REQUEST)
	  dct['description'] = ''
          #++ print '[%s.getLinkList]: %s'%(self.meta_type,self.id)
          rtn.extend([dct])
        return rtn
         
         
    # ---------------------------------------------------------------------------------------------
    #	ZMSFile.getMimeTypeIcon:
    #
    #	MIME-type icon.
    # ---------------------------------------------------------------------------------------------
    def getMimeTypeIcon(self,REQUEST):
        sContentType = getattr(self.getFile(REQUEST),'content_type',None)
        if not self.dctMimeType.has_key(sContentType):
            sContentType = 'content/unknown'
        return '<img src="%s" border="0" alt="%s">'%(self.dctMimeType[sContentType],sContentType)
                    

    # ---------------------------------------------------------------------------------------------
    # 	ZMSFile.getDataSizeStr: 
    #
    #	Display string for file-size.
    # ---------------------------------------------------------------------------------------------
    def getDataSizeStr(self, REQUEST):
      return self.getFile(REQUEST).getDataSizeStr()



    """
    ###############################################################################################    
    #
    #   Eigenschaften
    #
    ###############################################################################################    
    """
    
    ###############################################################################################
    #  ZMSFile.manage_changeProperties: 
    #
    #  Change file properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST): 
        """ ZMSFile.manage_changeProperties """
        
	message = ''
        if REQUEST.has_key('btn') and REQUEST['btn'] != self.getLangStr('BTN_CANCEL',manage_lang):
        
          ##### Object State ####
          self.setObjStateModified(REQUEST)
            
          ##### Properties ####
          for s_lang in self.getLanguages():
            # Set properties from Request-Object.
            if REQUEST.has_key('title_%s'%s_lang) and \
               REQUEST.has_key('titleshort_%s'%s_lang):
              self.setReqProperty('title',REQUEST)
              self.setReqProperty('titleshort',REQUEST)
              self.setReqProperty('active',REQUEST)
              self.setReqProperty('file',REQUEST)
              self.setReqProperty('counter',REQUEST)
              
          ##### Blobs ####
          _blobfields.synchronizeBlobFields(self)
          
          ##### Metadata ####
          self.setMetadata(lang,manage_lang,REQUEST)
        
          ##### VersionManager ####
          self.__autoCommit__(REQUEST)
          
	  ##### Message ####
          message = self.getLangStr('MSG_CHANGED',manage_lang)
          
        # Return with message.
        if hasattr(REQUEST,'RESPONSE'):
          if REQUEST.RESPONSE:
   	    self.checkIn(REQUEST)
            return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,urllib.quote(message),self.id))


    # ---------------------------------------------------------------------------------------------
    #	ZMSFile.isActive	
    #	ZMSFile.getTitle
    #	ZMSFile.getTitlealt
    #	ZMSFile.getFile
    #	ZMSFile.getContentType
    # ---------------------------------------------------------------------------------------------
    def isActive(self,REQUEST): return self.getObjProperty('active',REQUEST)
    def getTitle(self,REQUEST): return self.getObjProperty('title',REQUEST)
    def getTitlealt(self,REQUEST): return self.getObjProperty('titleshort',REQUEST)
    def getFile(self,REQUEST): return self.getObjProperty('file',REQUEST)
    def getContentType(self,REQUEST): return self.getFile(REQUEST).content_type

               
    """
    ###############################################################################################    
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################    
    """    
    
    # preload display interface
    rendershort_default = HTMLFile('dtml/zmsfile/render_default', globals()) 
    rendershort_print = HTMLFile('dtml/zmsfile/render_print', globals()) 
    
    ###############################################################################################    
    #  ZMSFile.renderShort:
    #
    #  Renders short presentation of File.
    ###############################################################################################    
    def renderShort(self,REQUEST):
      # Retrieve properties.
      file = self.getFile(REQUEST)
      url = None
      if file is not None:
        url = file.src(self,'file',REQUEST)
      # Return <html>-presentation.
      return self.rendershort_default(self,url=url,REQUEST=REQUEST)

    ###############################################################################################    
    # ZMSFile.printHtml:
    #
    # Renders print presentation of a File.
    ###############################################################################################    
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      # Retrieve properties.
      file = self.getFile(REQUEST)
      url = None
      if file is not None:
        url = file.src(self,'file',REQUEST)
      # Return <html>-presentation.
      return self.rendershort_print(self,url=url,REQUEST=REQUEST)

    ###############################################################################################    
    #	ZMSFile.catalogText:
    #
    #	Catalog text.
    ###############################################################################################    
    def catalogText(self, REQUEST):
      s = ""
      s = s + self.getTitle(REQUEST) + " "
      s = s + self.getTitlealt(REQUEST) + " " 
      return s
      
###################################################################################################
