###################################################################################################
# zmsobject.py
#
# Implementation of class ZMSObject (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile, Persistent
from DateTime.DateTime import DateTime
from Acquisition import Implicit
import AccessControl.Role
import urllib
import time
import string 
# Product Imports.
import _xmllib
import _copysupport
import _globals
import _language
import _versionmanager
import _cachemanager
import _metapatternmanager
import _linklib
import _exportable 
import _objattrs
import _objinputs
import _objtypes


###################################################################################################
###################################################################################################
###
###   C l a s s
###
###   This is the abstract base class for all ZMS-Objects.
###
###################################################################################################
###################################################################################################
class ZMSObject(
	Persistent,                     	# Persistent. 
	Implicit,                       	# Acquisition. 
	AccessControl.Role.RoleManager, 	# Security manager. 
	_language.Language,             	# Multilinguality.
	_versionmanager.VersionManager,		# Version manager.
	_copysupport.CopySupport,		# Copy Support (Paste Objects).
	_cachemanager.CacheableObject,		# Cacheable object
	_metapatternmanager.MetapatternObject,	# Metapattern object
	_exportable.Exportable,			# XML Export.
	_objattrs.ObjAttrs,			# Object-Attributes.
	_objinputs.ObjInputs,			# Object-Inputs.
	_objtypes.ObjTypes			# Object-Types
	): 

    # Documentation string.
    __doc__ = """ZMS product module."""
    # Version string. 
    __version__ = '0.1' 
    
    # Properties.
    # -----------
    MISC_ZMS = "misc_/zms/"
    kalender_gif = "misc_/zms/kalender.gif"
    banner_gif = "misc_/zms/banner.gif"
    spacer_gif = "misc_/zms/spacer.gif"
    mail_link_gif = "misc_/zms/mail_link.gif"
    external_link_gif = "misc_/zms/external_link.gif"
    internal_link_gif = "misc_/zms/internal_link.gif"
    internal_link_broken_gif = "misc_/zms/internal_link_broken.gif"
    btn_slctall_gif = "misc_/zms/btn_slctall.gif"
    btn_slctall_ON_gif = "misc_/zms/btn_slctall_on.gif"
    btn_slctnone_gif = "misc_/zms/btn_slctnone.gif"
    btn_slctnone_ON_gif = "misc_/zms/btn_slctnone_on.gif"
    btn_delete_gif = "misc_/zms/btn_delete.gif"
    btn_delete_ON_gif = "misc_/zms/btn_delete_on.gif"
    btn_delete_OFF_gif = "misc_/zms/btn_delete_off.gif"
    btn_cut_gif = "misc_/zms/btn_cut.gif"
    btn_cut_ON_gif = "misc_/zms/btn_cut_on.gif"
    btn_cut_OFF_gif = "misc_/zms/btn_cut_off.gif"
    btn_copy_gif = "misc_/zms/btn_copy.gif"
    btn_copy_ON_gif = "misc_/zms/btn_copy_on.gif"
    btn_copy_OFF_gif = "misc_/zms/btn_copy_off.gif"
    btn_paste_gif = "misc_/zms/btn_paste.gif"
    btn_paste_ON_gif = "misc_/zms/btn_paste_on.gif"
    btn_paste_OFF_gif = "misc_/zms/btn_paste_off.gif"

    # Templates.
    # ----------
    manage = HTMLFile('dtml/object/manage', globals())
    manage_workspace = HTMLFile('dtml/object/manage', globals())
    manage_menu = HTMLFile('dtml/object/manage_menu', globals())
    manage_tabs = HTMLFile('dtml/object/manage_tabs', globals()) # Manage Tabulators
    manage_bodyTop = HTMLFile('dtml/object/manage_bodytop', globals()) # Manage bodyTop: Seitenkopf (Logo und berschrift)
    manage_page_header = HTMLFile('dtml/object/manage_page_header', globals()) 
    manage_page_footer = HTMLFile('dtml/object/manage_page_footer', globals()) # Manage bodyBottom: Seitenfuss (Copyright)
    f_headline = HTMLFile('dtml/object/f_headline', globals()) # berschrift
    f_breadcrumbs = HTMLFile('dtml/object/f_breadcrumbs', globals()) # Manage Breadcrumbs
    f_css = HTMLFile('dtml/object/f_css', globals()) # CSS: Cascading StyleSheet
    f_css_sys = HTMLFile('dtml/object/f_css_sys', globals()) # CSS: Fixed System StyleSheet
    f_css_sys_ns = HTMLFile('dtml/object/f_css_sys_ns', globals()) # CSS: Fixed System StyleSheet (Netscape Navigator)
    f_css_printhtml = HTMLFile('dtml/object/f_css_printhtml', globals()) # CSS: Fixed PrintHTML StyleSheet
    f_submitBtn = HTMLFile('dtml/object/f_submitbtn', globals()) #
    f_languages = HTMLFile('dtml/object/f_languages', globals()) # Content Language select
    f_kalender = HTMLFile('dtml/object/f_kalender', globals()) 
    f_frame = HTMLFile('dtml/object/f_frame', globals()) 
    f_frame_top = HTMLFile('dtml/object/f_frame_top', globals()) 
    f_frame_bottom = HTMLFile('dtml/object/f_frame_bottom', globals()) 
    f_bo_area = HTMLFile('dtml/object/f_bo_area', globals()) 
    f_eo_area = HTMLFile('dtml/object/f_eo_area', globals()) 
    preview_html = HTMLFile('dtml/object/preview', globals()) 
    preview_top_html = HTMLFile('dtml/object/preview_top', globals()) 

    # JavaScript.
    # -----------
    comlib_js = HTMLFile('dtml/javascript/comlib', globals()) # Gemeinsame Funktionen
    formlib_js = HTMLFile('dtml/javascript/formlib', globals()) # Formfunktionen
    datelib_js = HTMLFile('dtml/javascript/datelib', globals()) # Datumsfunktionen
    fieldlib_js = HTMLFile('dtml/javascript/fieldlib', globals()) # Eingabefeldfunktionen

    # linklib.
    # --------
    js_browseObjs = HTMLFile('dtml/linklib/js_browseobjs', globals()) 
    browse_objs = HTMLFile('dtml/linklib/browse_objs', globals()) 


    ###############################################################################################
    # ZMSObject.__init__: 
    #
    # Constructor (initialise a new instance of ZMSObject).
    ###############################################################################################
    def __init__(self, id='', sort_id=0): 
      """ ZMSObject.__init__ """
      print id
      print sort_id
      self.id = id
      self.sort_id = _globals.format_sort_id(sort_id)

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.getSelf:
    # -------------------------------------------------------------------------------------------------
    def getSelf(self, meta_type=None):
      if meta_type is not None:
        if type(meta_type) is type([]):
          if self.meta_type not in meta_type and \
             self.getParentNode() is not None:
            return self.getParentNode().getSelf(meta_type)
        else:
          if self.meta_type != meta_type and \
             self.getParentNode() is not None:
            return self.getParentNode().getSelf(meta_type)
      return self

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.isPage:
    # -------------------------------------------------------------------------------------------------
    def isPage(self): return 0

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.isPageElement:
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self): return 0

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.isResource
    # -------------------------------------------------------------------------------------------------
    def isResource(self,REQUEST): return 0

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.isVisible:
    # ---------------------------------------------------------------------------------------------
    def isVisible(self, REQUEST):
      visible = 1
      visible = visible and self.isActive(REQUEST) # Object is active.
      visible = visible and self.getObjectDisplay(REQUEST) # Object has been committed.
      visible = visible and self.getDCCoverage(REQUEST) in ['obligation','proposal',REQUEST['lang']]
      return visible

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.display_type:
    # ---------------------------------------------------------------------------------------------
    def display_type(self, REQUEST=None, meta_type=''):
      if len(meta_type) == 0:
        meta_type = self.meta_type
      return self.getLangStr("TYPE_%s"%string.upper(meta_type),REQUEST['manage_lang'])

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.breadcrumbs_obj_path:
    # ---------------------------------------------------------------------------------------------
    def breadcrumbs_obj_path(self):
      rtn = []
      obj = self
      for lvl in range(0,self.getLevel()+1):
        obj_item = [obj]
        obj_item.extend(rtn)
        rtn = obj_item
        obj = obj.aq_parent
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.absolute_obj_path: DEPRECATED!
    # ---------------------------------------------------------------------------------------------
    def absolute_obj_path(self):
      ob = self.getDocumentElement()
      return '%s/%s/'%(ob.aq_parent.id,self.absolute_url()[len(ob.aq_parent.absolute_url())+1:])

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.relative_obj_path:
    # ---------------------------------------------------------------------------------------------
    def relative_obj_path(self):
      return self.absolute_url()[len(self.getDocumentElement().absolute_url())+1:]

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.raw_html_file:
    #
    #	Self-documentation. Key can be any HTMLFile defined for ZMS (e.g. bodyContent_Teaser)
    # ---------------------------------------------------------------------------------------------
    def raw_html_file(self, key, REQUEST, RESPONSE):
      """ raw_html_file """
      filename = getattr(self,key).raw
      f = open(filename,'r')
      html_file = f.read()
      f.close()
      html_file = _globals.html_quote(html_file)
      html_file = string.replace(html_file,'\n','<br>')
      html_file = string.replace(html_file,' ','&nbsp;')
      html_file = string.replace(html_file,'&lt;!--','<font style="color:green">&lt;!--')
      html_file = string.replace(html_file,'--&gt;','--&gt;</font>')
      html = ''
      html = html + '<html>'
      html = html + '<head><title>' + key + '</title></head>'
      html = html + '<body>'
      html = html + '<code>' + html_file + '</code>'
      html = html + '</body>'
      html = html + '</html>'
      return html

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.recurse_afterImport:
    #
    #	Remove helpers after XML-Import.
    # ---------------------------------------------------------------------------------------------
    def _recurse_afterImport(self):
      if hasattr(self,'oParent'): delattr(self,'oParent')
      if hasattr(self,'dTagStack'): delattr(self,'dTagStack')
      if hasattr(self,'dValueStack'): delattr(self,'dValueStack')
    #
    def recurse_afterImport(self):
      self._recurse_afterImport()


    """
    ###############################################################################################
    ###  
    ###  G l o b a l   F u n c t i o n s
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.rand_int:
    # ---------------------------------------------------------------------------------------------
    def rand_int(self, n):
      from whrandom import randint
      return randint(0,n)

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.url_quote:
    # ---------------------------------------------------------------------------------------------
    def url_quote(self, s):
      return urllib.quote(s)
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.url_inherit_params:
    # ---------------------------------------------------------------------------------------------
    def url_inherit_params(self, s, REQUEST, exclude=[]):
      rtn = s
      if REQUEST.form:
        count = 0
        for key in REQUEST.form.keys():
          if count == 0:
            rtn = rtn + "?"
          else:
            rtn = rtn + "&"
          rtn = rtn + key + "=" + urllib.quote(REQUEST.form[key])
          count = count + 1    	  
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.id_quote:
    # ---------------------------------------------------------------------------------------------
    def id_quote(self, s):
      return _globals.id_quote(s)

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.js_quote:
    # ---------------------------------------------------------------------------------------------
    def js_quote(self, s):
      return _globals.js_quote(s)

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.isPreviewRequest:
    # ---------------------------------------------------------------------------------------------
    def isPreviewRequest(self, REQUEST):
      return _globals.isPreviewRequest(REQUEST)


    """
    ###############################################################################################
    ###  
    ###  C o m m o n   F u n c t i o n s
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.recurse_updateVersion:
    #
    #	Update version.
    # ---------------------------------------------------------------------------------------------
    def recurse_updateVersion(self, lang, manage_lang, REQUEST): 
        message = ''

        # Process object.
        #++ print "[%s.recurse_updateVersion]: %s"%(self.meta_type,self.absolute_url())
        
        try: delattr(self,'lang')
        except: pass
        try: delattr(self,'manage_lang')
        except: pass
        try: delattr(self,'co_dat')
        except: pass
        try: delattr(self,'co_uid')
        except: pass
        
        # ---------------------------------------------------------------------
        # 19.12.2001 Build #039
        # Modification Time
        # ---------------------------------------------------------------------
        if not hasattr(self,'modification_time'):
          setattr(self,'modification_time',self.bobobase_modification_time())

        # ---------------------------------------------------------------------
        # 12.02.2002 Build #052
        # Multilingual Workflow
        # ---------------------------------------------------------------------
	if not hasattr(self,'__work_state__'):
      	  setattr(self,'__work_state__',_globals.MyClass())
	if not hasattr(self,'__work_version__'):
      	  setattr(self,'__work_version__',_globals.MyClass())
      	  self.cloneObjAttrs(self,self.__work_version__)

        # ---------------------------------------------------------------------
        # 18.02.2002 Build #058
        # Multilingual Modification User-ID and Time
        # ---------------------------------------------------------------------
        attrs = {'uid':'change_uid','modification_time':'change_dt'}
        for s_attr in attrs.keys():
          if hasattr(self,s_attr):
            for s_preview in [None,'preview']:
              req = {'preview':s_preview}
              ob = self.getObjVersion(req)
              for s_lang in self.getLanguages():
                #++ print "%s.%s_%s=%s"%(str(ob),attrs[s_attr],s_lang,str(getattr(self,s_attr)))
                setattr(ob,'%s_%s'%(attrs[s_attr],s_lang),getattr(self,s_attr))
            delattr(self,s_attr)
      
        # Return with message.
        return message


    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.findObjId:
    #
    #	Searches for Object identified by ID and returns the absolute URL of it.
    # ---------------------------------------------------------------------------------------------
    def _findObjId(self, id, REQUEST, management_interface=0):
      rtn = None
      if str(self.id) == str(id):
        rtn = self
      elif self.isPage():
        for child in self.filteredChildNodes(REQUEST,None,management_interface):
          if rtn is None:
            rtn = child._findObjId(id,REQUEST,management_interface)
      return rtn
    # --
    def findObjId(self, relative_obj_path, REQUEST, management_interface=0):
        root = self.getDocumentElement()
        # Root.
        if len(relative_obj_path) == 0:
          rtn = root
        else:
          # Trace relative object path.
          ob = root
          ids = string.split(relative_obj_path,"/")
          for id in ids:
            if ob is not None:
              ob_child = None
              for child in ob.filteredChildNodes(REQUEST,None,management_interface):
                if str(child.id) == str(id):
                  ob_child = child
              ob = ob_child
          rtn = ob
          # Find object by id.
          if rtn is None:
            rtn = root._findObjId(ids[-1],REQUEST,management_interface) 
        # Return object.
        return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.href2Html:
    #	ZMSObject.getHref2IndexHtml:
    #	ZMSObject.getHref2SitemapHtml:
    #
    #   "Sans-Document"-Navigation: reference to first ZMSContainerObject
    #	that contains visible page-elements.
    # ---------------------------------------------------------------------------------------------
    def getHref2Html(self, fct, REQUEST):
      href = self.absolute_url()
      href = href + '/%s_%s.html'%(fct,REQUEST['lang'])
      if _globals.isPreviewRequest(REQUEST) and not _globals.isCacheRequest(REQUEST):
        href = href + '?preview=preview'
      return href
    #++
    def findHref2Html(self, fct, REQUEST):
      if not self.isPage():
        prnt = self.getParentNode()
        return prnt.getHref2Html(fct,REQUEST)
      else:
        cntnr = None
        for ob in self.getChildNodes(REQUEST):
          if ob.isVisible(REQUEST): 
            if ob.isPageElement():
              return self.getHref2Html(fct,REQUEST)
            elif ob.isPage():
              if cntnr is None: cntnr = ob
        if cntnr is not None:
          return cntnr.findHref2Html(fct,REQUEST)
      return self.getHref2Html(fct,REQUEST)
    #++
    def getHref2IndexHtml(self, REQUEST): 
      if not REQUEST.has_key('lang'): REQUEST.set('lang',self.getPrimaryLanguage())
      return self.findHref2Html('index',REQUEST)
    #++
    def getHref2SitemapHtml(self, REQUEST): 
      if not REQUEST.has_key('lang'): REQUEST.set('lang',self.getPrimaryLanguage())
      return self.findHref2Html('sitemap',REQUEST)
    #++
    def getHref2PrintHtml(self, REQUEST): 
      if not REQUEST.has_key('lang'): REQUEST.set('lang',self.getPrimaryLanguage())
      return self.findHref2Html('index_print',REQUEST) + "?" + REQUEST['QUERY_STRING']


    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getActions:
    #
    #	Object-actions of management interface.
    # ---------------------------------------------------------------------------------------------
    def getActions(self, REQUEST):
      rtn = []
      lang = REQUEST['lang']
      manage_lang = REQUEST['manage_lang']
      if self.getAutocommit() or \
         self.getPrimaryLanguage() == lang or \
         self.getDCCoverage(REQUEST) not in ['obligation','proposal']:
        rtn.append(('----- ' + self.getLangStr('ACTION0',manage_lang) + ' -----',''))
        rtn.append((self.getLangStr('BTN_DELETE',manage_lang),'manage_deleteObjs'))
        rtn.append((self.getLangStr('BTN_CUT',manage_lang),'manage_cutObjects'))
        rtn.append((self.getLangStr('BTN_COPY',manage_lang),'manage_copyObjects'))
        if self.cb_dataValid():
          rtn.append((self.getLangStr('BTN_PASTE',manage_lang),'manage_pasteObjs'))
        rtn.append((self.getLangStr('ACTION_MOVEUP',manage_lang),self.id + '/manage_moveObjUp'))
        rtn.append((self.getLangStr('ACTION_MOVEDOWN',manage_lang),self.id + '/manage_moveObjDown'))
      rtn.extend(self.getParentNode().filtered_insert_actions(self.id + '/',REQUEST))
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getMailHost:
    #
    #	Returns MailHost-object (None if there is no MailHost).
    # ---------------------------------------------------------------------------------------------
    def getMailHost(self):
      root = self.getDocumentElement().aq_parent
      if len(root.objectValues(['Mail Host']))==1:
        return root.objectValues(['Mail Host'])[0]
      return None


    """
    ###############################################################################################
    ###  
    ###  Check In / Out
    ### 
    ###############################################################################################
    """

    checkout_overtake_html = HTMLFile('dtml/object/checkout_overtake', globals()) 
    checkout = HTMLFile('dtml/object/checkout', globals()) 

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.checkIn:
    # ---------------------------------------------------------------------------------------------
    def checkIn(self, REQUEST):
      if hasattr(self,'co_dat_%s'%REQUEST['lang']): delattr(self,'co_dat_%s'%REQUEST['lang'])
      if hasattr(self,'co_uid_%s'%REQUEST['lang']): delattr(self,'co_uid_%s'%REQUEST['lang'])

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.checkOut:
    # ---------------------------------------------------------------------------------------------
    def checkOut(self, REQUEST):
      setattr(self,'co_dat_%s'%REQUEST['lang'],time.time())
      setattr(self,'co_uid_%s'%REQUEST['lang'],str(REQUEST['AUTHENTICATED_USER']))

    """
    ###############################################################################################
    ###  
    ###  Sitemap
    ### 
    ###############################################################################################
    """

    ###############################################################################################
    #  ZMSContainerObject.manage_sitemap: 
    #
    #  De-/Activate sitemap.
    ###############################################################################################
    def manage_sitemap(self, lang, manage_lang, REQUEST, RESPONSE):
      """ ZMSObject.manage_sitemap """
      sitemap = 1
      if REQUEST.has_key('dtpref_sitemap'):
        sitemap = not string.atoi(REQUEST['dtpref_sitemap'])
      e=(DateTime('GMT') + 365).rfc822()
      RESPONSE.setCookie('dtpref_sitemap',str(sitemap),path='/',expires=e)
      return RESPONSE.redirect('manage?lang=%s&manage_lang=%s'%(lang,manage_lang))


    """
    ###############################################################################################
    ###  
    ###  Related Links 
    ### 
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getLinkObj:
    #
    #	Resolves internal/external links and returns Object.
    # ---------------------------------------------------------------------------------------------
    def getLinkObj(self, url, REQUEST): 
      return _linklib.getLinkObj(self,url,REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getLinkUrl:
    #
    #	Resolves internal/external links and returns URL.
    # ---------------------------------------------------------------------------------------------
    def getLinkUrl(self, url, REQUEST): 
      return _linklib.getLinkUrl(self,url,REQUEST)


    """
    ###############################################################################################    
    ###  
    ###  Catalog (Suche)
    ### 
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getCatalogNavUrl:
    #
    #	Returns catalog-navigation url.
    # ---------------------------------------------------------------------------------------------
    def getCatalogNavUrl(self,REQUEST):
      rtn = REQUEST['URL']
      rtn = '%s?op=%s'%(rtn,urllib.quote(REQUEST['op']))
      rtn = '%s&raw=%s'%(rtn,urllib.quote(REQUEST['raw']))
      rtn = '%s&option=%s'%(rtn,urllib.quote(REQUEST['option']))
      rtn = '%s&orderby=%s'%(rtn,urllib.quote(str(REQUEST['orderby'])))
      return rtn


    """
    ###############################################################################################    
    ###  
    ###  DOM-Methoden 
    ### 
    ###############################################################################################    
    """    

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getLevel:
    #
    #	The hierarchical level of this node.
    # ---------------------------------------------------------------------------------------------
    def getLevel(self, level=0):
      parent = self.getParentNode()
      if parent is not None:
        level = parent.getLevel(level+1)
      return level

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.isAnchestor:
    #
    #	True if this object is anchestor of given object.
    # ---------------------------------------------------------------------------------------------
    def isAnchestor(self, ob):
      return string.find(ob.absolute_url() + "/",self.absolute_url() + "/") == 0

    # ---------------------------------------------------------------------------------------------
    #  ZMSObject.getParentByDepth:
    #
    #  The parent of this node by depth. 
    # ---------------------------------------------------------------------------------------------
    def getParentByDepth(self, deep):
        rtn = self
        for i in range(0,deep):
          rtn = rtn.getParentNode()
        return rtn

    # ---------------------------------------------------------------------------------------------
    #  ZMSObject.getParentByLevel:
    #
    #  The parent of this node by level. 
    # ---------------------------------------------------------------------------------------------
    def getParentByLevel(self, level):
        rtn = self
        while rtn.getLevel() > level:
          rtn = rtn.getParentNode()
        return rtn

    # ---------------------------------------------------------------------------------------------
    #  ZMSObject.getParentNode:
    #
    #  The parent of this node. 
    #  All nodes except the root object may have a parent.
    # ---------------------------------------------------------------------------------------------
    def getParentNode(self):
        parent = None
        if hasattr(self,'aq_parent'):
          parent = getattr(self,'aq_parent')
        return parent

    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getChildNodes:
    #
    #	Returns a NodeList that contains all children of this node, 
    #	if none, this is a empty NodeList. 
    # ---------------------------------------------------------------------------------------------
    def getChildNodes(self, REQUEST=None, meta_types=None): 
      return []


    """
    ###############################################################################################    
    ###  
    ###  Sortier-Reihenfolge
    ### 
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    # ZMSObject.getSortId:
    #
    # Returns sort-ID (integer)
    # ---------------------------------------------------------------------------------------------
    def getSortId(self,REQUEST): 
      #++ print "[%s.getSortId]: %s=%s"%(self.meta_type,self.id,self.getObjProperty('sort_id',REQUEST))
      return string.atoi(self.getObjProperty('sort_id',REQUEST)[1:])


    ###############################################################################################    
    # ZMSObject.manage_moveObjUp:
    #
    # Moves an Object up in sort order.
    ###############################################################################################    
    def manage_moveObjUp(self, lang, manage_lang, REQUEST): 
      """ ZMSObject.manage_moveObjUp """
      parent = self.getParentNode()
      sort_id = self.getSortId(REQUEST)
      self.setObjStateModified(REQUEST)
      self.setObjProperty('sort_id',_globals.format_sort_id(sort_id - 15))
      parent.normalizeSortIds()
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_MOVEDOBJUP',manage_lang)%("<i>%s</i>"%self.display_type(REQUEST)))
      REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(parent.absolute_url(),lang,manage_lang,message,self.id))


    ###############################################################################################    
    # ZMSObject.manage_moveObjDown:
    #
    # Moves an Object down in sort order.
    ###############################################################################################    
    def manage_moveObjDown(self, lang, manage_lang, REQUEST): 
      """ ZMSObject.manage_moveObjDown """
      parent = self.getParentNode()
      sort_id = self.getSortId(REQUEST)
      self.setObjStateModified(REQUEST)
      self.setObjProperty('sort_id',_globals.format_sort_id(sort_id + 15))
      parent.normalizeSortIds()
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%self.display_type(REQUEST)))
      REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(parent.absolute_url(),lang,manage_lang,message,self.id))


    """
    ###############################################################################################    
    #
    #   Darstellung
    #
    ###############################################################################################    
    """

    ###############################################################################################    
    #  ZMSObject.getBodyContent:
    #         
    #  HTML presentation in BodyContent. 
    ###############################################################################################    
    #++
    def _getBodyContent(self, REQUEST, thumbnail=0): 
      rtn = ""
      return rtn
    #++
    def getBodyContent(self, REQUEST):
      rtn = ""
      if self.isVisible(REQUEST):
        rtn = rtn + '<a name="%s"></a>'%self.id
        rtn = rtn + self._getBodyContent(REQUEST)
      return rtn

    ###############################################################################################    
    # ZMSObject.printHtml:
    #
    # Renders print presentation of a ContainerObject.
    ###############################################################################################    
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      return self._getBodyContent(REQUEST)


    """
    ###############################################################################################    
    #
    #   BUILDER
    #
    ###############################################################################################    
    """
    
    ###############################################################################################
    # ZMSObject.xmlOnStartElement(self, dTagName, dAttrs, oCurrNodes, oRoot):
    # ZMSObject.xmlOnCharacterData(self, data, bInCData):
    # ZMSObject.xmlOnEndElement(self):
    # ZMSObject.xmlOnUnknownStartTag(self, sTagName, dTagAttrs)
    # ZMSObject.xmlOnUnknownEndTag(self, sTagName)
    # ZMSObject.xmlGetTagName(self):
    # ZMSObject.xmlGetParent(self):
    #
    # handler for XML-Builder (_builder.py)
    ###############################################################################################
    def xmlOnStartElement(self, sTagName, dTagAttrs, oParentNode, oRoot):
        _globals.myDebugger.write("[%s.xmlOnStartElement]: sTagName=%s"%(self.meta_type,sTagName))
        
        self.dTagStack    = _globals.MyStack()
        self.dValueStack  = _globals.MyStack()
        
	#-- ATTRIBUTES
	for i in range(0,len(dTagAttrs)/2):
	  key = dTagAttrs[i*2]
	  if key != 'id':
  	    value = dTagAttrs[i*2+1]
	    #++ print "[%s.xmlOnStartElement]: %s=%s"%(self.meta_type,str(key),str(value))
	    setattr(self,key,value)
        
        # WORKAROUND! The member variable "aq_parent" does not contain the right parent object at 
        # this stage of the creation process (it will later on!). Therefore, we introduce a special
        # attribute containing the parent object, which will be used by xmlGetParent() (see below).
        self.oParent = oParentNode


    def xmlOnEndElement(self): 
        pass


    def xmlOnCharacterData(self, sData, bInCData):
        _xmllib.xmlOnCharacterData(self,sData,bInCData)
        pass


    def xmlOnUnknownStartTag(self, sTagName, dTagAttrs):
        return _xmllib.xmlOnUnknownStartTag(self,sTagName,dTagAttrs)


    def xmlOnUnknownEndTag(self, sTagName):
        return _xmllib.xmlOnUnknownEndTag(self,sTagName)


    def xmlGetParent(self):
        return self.oParent


    def xmlGetTagName(self):
        return self.meta_type

###################################################################################################